/*
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.agent.jcmd.backend.internal;

import com.redhat.thermostat.agent.http.HttpRequestService;
import com.redhat.thermostat.agent.utils.management.MXBeanConnectionPool;
import com.redhat.thermostat.backend.Backend;
import com.redhat.thermostat.backend.BaseBackend;
import com.redhat.thermostat.commands.agent.receiver.RequestReceiver;

import com.redhat.thermostat.common.config.experimental.ConfigurationInfoSource;
import com.redhat.thermostat.jvm.overview.agent.model.VmMapperService;
import com.redhat.thermostat.lang.schema.JSONService;
import org.apache.felix.scr.annotations.Activate;
import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.Deactivate;
import org.apache.felix.scr.annotations.Reference;
import org.apache.felix.scr.annotations.Service;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;

import java.util.Dictionary;
import java.util.Hashtable;

@Component
@Service(value = Backend.class)
public class JCMDBackend extends BaseBackend {

    private boolean enabled = false;

    private ServiceRegistration<RequestReceiver> reg;
    private JMXReceiver receiver;
    private JMXConnectionManager connectionManager;

    private BundleContext context;

    @Reference
    private MXBeanConnectionPool mxBeanPool;

    @Reference
    private JSONService json;

    @Reference
    private HttpRequestService httpRequestService;

    @Reference
    private ConfigurationInfoSource configInfoSource;

    @Reference
    private VmMapperService mapperService;

    public JCMDBackend() {
        super("JCMD Stats Collector Backend",
              "Gathers stats from JVM with diagnostic options enabled",
              "Red Hat, Inc.");
    }

    @Activate
    void _activate_(BundleContext context) {
        this.context = context;
        if (connectionManager == null) {
            connectionManager = new JMXConnectionManager(mxBeanPool);
            NetworkHandler networkHandler = new NetworkHandler(httpRequestService, configInfoSource);
            receiver = new JMXReceiver(connectionManager, json, mapperService, networkHandler);
        }
    }

    @Deactivate
    private void _deactivate_(BundleContext context) {
        if (reg != null) {
            reg.unregister();
            reg = null;
        }
    }

    @Override
    public boolean activate() {
        enabled = true;

        if (reg == null) {
            Dictionary<String, Object> serviceProperties = new Hashtable<>();
            serviceProperties.put("servicename", "jcmd");
            reg = context.registerService(RequestReceiver.class, receiver, serviceProperties);
        }

        return true;
    }

    @Override
    public boolean deactivate() {
        enabled = false;
        _deactivate_(this.context);
        return true;
    }

    @Override
    public boolean isActive() {
        return enabled;
    }

}
