/*
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.agent.jcmd.backend.internal;

import com.redhat.thermostat.commands.agent.receiver.RequestReceiver;
import com.redhat.thermostat.commands.model.AgentRequest;
import com.redhat.thermostat.commands.model.WebSocketResponse;
import com.redhat.thermostat.common.utils.LoggingUtils;
import com.redhat.thermostat.agent.jcmd.backend.internal.model.GCStatsModel;
import com.redhat.thermostat.agent.jcmd.backend.internal.model.JCMDBinding;
import com.redhat.thermostat.jvm.overview.agent.model.VmMapperService;
import com.redhat.thermostat.lang.schema.JSONService;
import com.redhat.thermostat.lang.schema.models.Pid;
import com.redhat.thermostat.lang.schema.models.Timestamp;

import java.util.logging.Level;
import java.util.logging.Logger;

class JMXReceiver implements RequestReceiver {

    private static final Logger log = LoggingUtils.getLogger(JMXReceiver.class);

    private JMXConnectionManager connectionManager;
    private JSONService json;
    private NetworkHandler networkHandler;
    private VmMapperService mapperService;

    public JMXReceiver(JMXConnectionManager connectionManager, JSONService json,
                       VmMapperService mapperService, NetworkHandler networkHandler)
    {
        this.connectionManager = connectionManager;
        this.json = json;
        this.networkHandler = networkHandler;
        this.mapperService = mapperService;
    }

    @Override
    public WebSocketResponse receive(AgentRequest request) {

        WebSocketResponse.ResponseType responseType = WebSocketResponse.ResponseType.ERROR;
        String vmId = request.getJvmId();
        Pid pid = mapperService.getPid(vmId);
        if (pid == null) {
            return new WebSocketResponse(request.getSequenceId(), responseType);
        }

        try {
            JCMDBinding binding = connectionManager.getBindings(pid);

            Timestamp timestamp = new Timestamp(System.currentTimeMillis());
            String gcStats = binding.getGCStats();

            // enable for debugging, this prints a lot of output
//            log.log(Level.WARNING, "gcStats follow ----------");
//            log.log(Level.WARNING, gcStats);
//            log.log(Level.WARNING, "gcStats follow end ------");

            GCStatsModel model = new GCStatsModel(timestamp, gcStats);
            String serialisedModel = json.serialiase(model);

            responseType = networkHandler.send(serialisedModel, request.getSystemId(), vmId);

        } catch (Exception ex) {
            log.log(Level.WARNING, "Error!" , ex);
            responseType = WebSocketResponse.ResponseType.ERROR;
        }

        return new WebSocketResponse(request.getSequenceId(), responseType);
    }
}
