/*
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.agent.jcmd.backend.internal;

import com.redhat.thermostat.agent.http.HttpRequestService;
import com.redhat.thermostat.commands.model.WebSocketResponse;
import com.redhat.thermostat.common.config.experimental.ConfigurationInfoSource;
import com.redhat.thermostat.common.plugin.PluginConfiguration;

import java.io.IOException;
import java.net.URI;
import java.util.logging.Level;
import java.util.logging.Logger;

public class NetworkHandler {
    private static final String PLUGIN_ID = "jcmd";
    private static final String SYSTEM_PATH = "systems/";
    private static final String VM_PATH = "/jvms/";

    private HttpRequestService httpRequestService;

    private PluginConfiguration configuration;

    public NetworkHandler(HttpRequestService httpRequestService, ConfigurationInfoSource configInfoSource)
    {
        this.httpRequestService = httpRequestService;
        this.configuration = new ConfigurationCreator().create(configInfoSource);
    }

    private URI getURI(String systemId, String vmId) throws IOException {
        StringBuilder builder = new StringBuilder();
        builder.append(SYSTEM_PATH);
        builder.append(systemId);
        builder.append(VM_PATH);
        builder.append(vmId);
        return configuration.getGatewayURL().resolve(builder.toString());
    }

    public WebSocketResponse.ResponseType send(String serialisedModel, String systemId, String vmId) throws Exception {
        //Logger.getLogger(this.getClass().getName()).log(Level.WARNING, "jvmid: " + vmId + ", body: " + serialisedModel);
        httpRequestService.sendHttpRequest(serialisedModel, getURI(systemId, vmId), HttpRequestService.Method.POST);
        return WebSocketResponse.ResponseType.OK;
    }

    static class ConfigurationCreator {

        PluginConfiguration create(ConfigurationInfoSource source) {
            return new PluginConfiguration(source, PLUGIN_ID);
        }

    }
}
